@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2018 Ben Collins <ben@cyphre.com>
+/* Copyright (C) 2015-2022 Ben Collins <bcollins@maclara-llc.com>
    This file is part of the JWT C Library
 
    This Source Code Form is subject to the terms of the Mozilla Public
@@ -98,6 +98,12 @@ const char *jwt_alg_str(jwt_alg_t alg)
 		return "ES384";
 	case JWT_ALG_ES512:
 		return "ES512";
+	case JWT_ALG_PS256:
+		return "PS256";
+	case JWT_ALG_PS384:
+		return "PS384";
+	case JWT_ALG_PS512:
+		return "PS512";
 	default:
 		return NULL;
 	}
@@ -128,6 +134,12 @@ jwt_alg_t jwt_str_alg(const char *alg)
 		return JWT_ALG_ES384;
 	else if (!strcmp(alg, "ES512"))
 		return JWT_ALG_ES512;
+	else if (!strcmp(alg, "PS256"))
+		return JWT_ALG_PS256;
+	else if (!strcmp(alg, "PS384"))
+		return JWT_ALG_PS384;
+	else if (!strcmp(alg, "PS512"))
+		return JWT_ALG_PS512;
 
 	return JWT_ALG_INVAL;
 }
@@ -177,7 +189,7 @@ int jwt_set_alg(jwt_t *jwt, jwt_alg_t alg, const unsigned char *key, int len)
 	return 0;
 }
 
-jwt_alg_t jwt_get_alg(jwt_t *jwt)
+jwt_alg_t jwt_get_alg(const jwt_t *jwt)
 {
 	return jwt->alg;
 }
@@ -416,25 +428,30 @@ void jwt_base64uri_encode(char *str)
 	str[t] = '\0';
 }
 
-static int jwt_sign(jwt_t *jwt, char **out, unsigned int *len, const char *str)
+static int jwt_sign(jwt_t *jwt, char **out, unsigned int *len, const char *str, unsigned int str_len)
 {
 	switch (jwt->alg) {
 	/* HMAC */
 	case JWT_ALG_HS256:
 	case JWT_ALG_HS384:
 	case JWT_ALG_HS512:
-		return jwt_sign_sha_hmac(jwt, out, len, str);
+		return jwt_sign_sha_hmac(jwt, out, len, str, str_len);
 
 	/* RSA */
 	case JWT_ALG_RS256:
 	case JWT_ALG_RS384:
 	case JWT_ALG_RS512:
 
+	/* PS */
+	case JWT_ALG_PS256:
+	case JWT_ALG_PS384:
+	case JWT_ALG_PS512:
+
 	/* ECC */
 	case JWT_ALG_ES256:
 	case JWT_ALG_ES384:
 	case JWT_ALG_ES512:
-		return jwt_sign_sha_pem(jwt, out, len, str);
+		return jwt_sign_sha_pem(jwt, out, len, str, str_len);
 
 	/* You wut, mate? */
 	default:
@@ -442,25 +459,30 @@ static int jwt_sign(jwt_t *jwt, char **out, unsigned int *len, const char *str)
 	}
 }
 
-static int jwt_verify(jwt_t *jwt, const char *head, const char *sig)
+static int jwt_verify(jwt_t *jwt, const char *head, unsigned int head_len, const char *sig)
 {
 	switch (jwt->alg) {
 	/* HMAC */
 	case JWT_ALG_HS256:
 	case JWT_ALG_HS384:
 	case JWT_ALG_HS512:
-		return jwt_verify_sha_hmac(jwt, head, sig);
+		return jwt_verify_sha_hmac(jwt, head, head_len, sig);
 
 	/* RSA */
 	case JWT_ALG_RS256:
 	case JWT_ALG_RS384:
 	case JWT_ALG_RS512:
 
+	/* PS */
+	case JWT_ALG_PS256:
+	case JWT_ALG_PS384:
+	case JWT_ALG_PS512:
+
 	/* ECC */
 	case JWT_ALG_ES256:
 	case JWT_ALG_ES384:
 	case JWT_ALG_ES512:
-		return jwt_verify_sha_pem(jwt, head, sig);
+		return jwt_verify_sha_pem(jwt, head, head_len, sig);
 
 	/* You wut, mate? */
 	default:
@@ -484,6 +506,8 @@ static int jwt_parse_body(jwt_t *jwt, char *body)
 
 static int jwt_parse_head(jwt_t *jwt, char *head)
 {
+	const char *alg;
+
 	if (jwt->headers) {
 		json_decref(jwt->headers);
 		jwt->headers = NULL;
@@ -493,24 +517,18 @@ static int jwt_parse_head(jwt_t *jwt, char *head)
 	if (!jwt->headers)
 		return EINVAL;
 
+	alg = get_js_string(jwt->headers, "alg");
+	jwt->alg = jwt_str_alg(alg);
+	if (jwt->alg == JWT_ALG_INVAL) {
+		return EINVAL;
+	}
+
 	return 0;
 }
 
-static int jwt_verify_head(jwt_t *jwt, char *head)
+static int jwt_verify_head(jwt_t *jwt)
 {
 	int ret = 0;
-	if ((ret = jwt_parse_head(jwt, head))) {
-		return ret;
-	}
-
-	const char *val;
-
-	val = get_js_string(jwt->headers, "alg");
-	jwt->alg = jwt_str_alg(val);
-	if (jwt->alg == JWT_ALG_INVAL) {
-		ret = EINVAL;
-		goto verify_head_done;
-	}
 
 	if (jwt->alg != JWT_ALG_NONE) {
 		if (jwt->key) {
@@ -521,20 +539,17 @@ static int jwt_verify_head(jwt_t *jwt, char *head)
 		}
 	} else {
 		/* If alg is NONE, there should not be a key */
-		if (jwt->key){
+		if (jwt->key) {
 			ret = EINVAL;
 		}
 	}
 
-verify_head_done:
-
 	return ret;
 }
 
-int jwt_decode(jwt_t **jwt, const char *token, const unsigned char *key,
-	       int key_len)
+static int jwt_parse(jwt_t **jwt, const char *token, unsigned int *len)
 {
-	char *head = jwt_strdup(token);
+	char *head = NULL;
 	jwt_t *new = NULL;
 	char *body, *sig;
 	int ret = EINVAL;
@@ -543,6 +558,7 @@ int jwt_decode(jwt_t **jwt, const char *token, const unsigned char *key,
 		return EINVAL;
 
 	*jwt = NULL;
+	head = jwt_strdup(token);
 
 	if (!head)
 		return ENOMEM;
@@ -550,7 +566,7 @@ int jwt_decode(jwt_t **jwt, const char *token, const unsigned char *key,
 	/* Find the components. */
 	for (body = head; body[0] != '.'; body++) {
 		if (body[0] == '\0')
-			goto decode_done;
+			goto parse_done;
 	}
 
 	body[0] = '\0';
@@ -558,52 +574,132 @@ int jwt_decode(jwt_t **jwt, const char *token, const unsigned char *key,
 
 	for (sig = body; sig[0] != '.'; sig++) {
 		if (sig[0] == '\0')
-			goto decode_done;
+			goto parse_done;
 	}
 
 	sig[0] = '\0';
-	sig++;
 
 	/* Now that we have everything split up, let's check out the
 	 * header. */
 	ret = jwt_new(&new);
 	if (ret) {
-		goto decode_done;
+		goto parse_done;
 	}
 
+	if ((ret = jwt_parse_head(new, head))) {
+		goto parse_done;
+	}
+
+	ret = jwt_parse_body(new, body);
+parse_done:
+	if (ret) {
+		jwt_free(new);
+        	*jwt = NULL;
+	} else {
+		*jwt = new;
+		*len = sig - head;
+	}
+
+	jwt_freemem(head);
+
+	return ret;
+}
+
+static int jwt_copy_key(jwt_t *jwt, const unsigned char *key, int key_len)
+{
+	int ret = 0;
+
+	if (key_len) {
+		jwt->key = jwt_malloc(key_len);
+		if (jwt->key == NULL)
+			return ENOMEM;
+		memcpy(jwt->key, key, key_len);
+		jwt->key_len = key_len;
+	}
+
+	return ret;
+}
+
+int jwt_decode(jwt_t **jwt, const char *token, const unsigned char *key,
+	       int key_len)
+{
+	jwt_t *new = NULL;
+	int ret = EINVAL;
+	unsigned int payload_len;
+
+	ret = jwt_parse(jwt, token, &payload_len);
+	if (ret) {
+		return ret;
+	}
+	new = *jwt;
+
 	/* Copy the key over for verify_head. */
-	if (key_len) {
-		new->key = jwt_malloc(key_len);
-		if (new->key == NULL)
-			goto decode_done;
-		memcpy(new->key, key, key_len);
-		new->key_len = key_len;
-	}
-
-	ret = jwt_verify_head(new, head);
+	ret = jwt_copy_key(new, key, key_len);
 	if (ret)
 		goto decode_done;
 
-	ret = jwt_parse_body(new, body);
+	ret = jwt_verify_head(new);
 	if (ret)
 		goto decode_done;
 
 	/* Check the signature, if needed. */
 	if (new->alg != JWT_ALG_NONE) {
-		/* Re-add this since it's part of the verified data. */
-		body[-1] = '.';
-		ret = jwt_verify(new, head, sig);
+		const char *sig = token + (payload_len + 1);
+		ret = jwt_verify(new, token, payload_len, sig);
 	} else {
 		ret = 0;
 	}
 
 decode_done:
-	if (ret)
+	if (ret) {
 		jwt_free(new);
-	else
-		*jwt = new;
+		*jwt = NULL;
+	}
 
-	jwt_freemem(head);
+	return ret;
+}
+
+int jwt_decode_2(jwt_t **jwt, const char *token, jwt_key_p_t key_provider) 
+{
+	jwt_t *new = NULL;
+	int ret = EINVAL;
+	unsigned int payload_len;
+	jwt_key_t key;
+
+	ret = jwt_parse(jwt, token, &payload_len);
+	if (ret) {
+		return ret;
+	}
+	new = *jwt;
+
+	/* Obtain the key. */
+	if (new->alg != JWT_ALG_NONE) {
+		ret = key_provider(new, &key);
+		if (ret) {
+			goto decode_done;
+		}
+		ret = jwt_copy_key(new, key.jwt_key, key.jwt_key_len);
+		if (ret)
+			goto decode_done;
+	}
+
+	ret = jwt_verify_head(new);
+	if (ret)
+		goto decode_done;
+
+	/* Check the signature, if needed. */
+	if (new->alg != JWT_ALG_NONE) {
+		ret = jwt_verify(new, token, payload_len,
+				 token + (payload_len + 1));
+	} else {
+		ret = 0;
+	}
+
+decode_done:
+	if (ret) {
+		jwt_free(new);
+		*jwt = NULL;
+	}
 
 	return ret;
 }
@@ -741,25 +837,6 @@ int jwt_del_grants(jwt_t *jwt, const char *grant)
 	return 0;
 }
 
-#ifdef _MSC_VER
-
-int jwt_del_grant(jwt_t *jwt, const char *grant);
-#pragma comment(linker, "/alternatename:jwt_del_grant=jwt_del_grants")
-
-#else
-
-#ifdef NO_WEAK_ALIASES
-int jwt_del_grant(jwt_t *jwt, const char *grant)
-{
-	return jwt_del_grants(jwt, grant);
-}
-#else
-int jwt_del_grant(jwt_t *jwt, const char *grant)
-	__attribute__ ((weak, alias ("jwt_del_grants")));
-#endif
-
-#endif /* _MSC_VER */
-
 const char *jwt_get_header(jwt_t *jwt, const char *header)
 {
 	if (!jwt || !header || !strlen(header)) {
@@ -1093,7 +1170,7 @@ static int jwt_encode(jwt_t *jwt, char **out)
 	}
 
 	/* Now the signature. */
-	ret = jwt_sign(jwt, &sig, &sig_len, buf);
+	ret = jwt_sign(jwt, &sig, &sig_len, buf, strlen(buf));
 	jwt_freemem(buf);
 
 	if (ret)
@@ -1193,6 +1270,9 @@ int jwt_valid_new(jwt_valid_t **jwt_valid, jwt_alg_t alg)
 
 	(*jwt_valid)->status = JWT_VALIDATION_ERROR;
 
+	(*jwt_valid)->nbf_leeway = 0;
+	(*jwt_valid)->exp_leeway = 0;
+
 	(*jwt_valid)->req_grants = json_object();
 	if (!(*jwt_valid)->req_grants) {
 		jwt_freemem(*jwt_valid);
@@ -1221,6 +1301,22 @@ unsigned int jwt_valid_get_status(jwt_valid_t *jwt_valid)
 	return jwt_valid->status;
 }
 
+time_t jwt_valid_get_nbf_leeway(jwt_valid_t *jwt_valid)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	return jwt_valid->nbf_leeway;
+}
+
+time_t jwt_valid_get_exp_leeway(jwt_valid_t *jwt_valid)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	return jwt_valid->exp_leeway;
+}
+
 int jwt_valid_add_grant(jwt_valid_t *jwt_valid, const char *grant, const char *val)
 {
 	if (!jwt_valid || !grant || !strlen(grant) || !val)
@@ -1349,6 +1445,26 @@ int jwt_valid_set_now(jwt_valid_t *jwt_valid, const time_t now)
 	return 0;
 }
 
+int jwt_valid_set_nbf_leeway(jwt_valid_t *jwt_valid, const time_t nbf_leeway)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	jwt_valid->nbf_leeway = nbf_leeway;
+
+	return 0;
+}
+
+int jwt_valid_set_exp_leeway(jwt_valid_t *jwt_valid, const time_t exp_leeway)
+{
+	if (!jwt_valid)
+		return EINVAL;
+
+	jwt_valid->exp_leeway = exp_leeway;
+
+	return 0;
+}
+
 int jwt_valid_set_headers(jwt_valid_t *jwt_valid, int hdr)
 {
 	if (!jwt_valid)
@@ -1399,12 +1515,12 @@ unsigned int jwt_validate(jwt_t *jwt, jwt_valid_t *jwt_valid)
 
 	/* Validate expires */
 	t = get_js_int(jwt->grants, "exp");
-	if (jwt_valid->now && t != -1 && jwt_valid->now >= t)
+	if (jwt_valid->now && t != -1 && jwt_valid->now - jwt_valid->exp_leeway >= t)
 		jwt_valid->status |= JWT_VALIDATION_EXPIRED;
 
 	/* Validate not-before */
 	t = get_js_int(jwt->grants, "nbf");
-	if (jwt_valid->now && t != -1 && jwt_valid->now < t)
+	if (jwt_valid->now && t != -1 && jwt_valid->now + jwt_valid->nbf_leeway < t)
 		jwt_valid->status |= JWT_VALIDATION_TOO_NEW;
 
 	/* Validate replicated issuer */
